﻿
Imports MI = System.Reflection.MethodInfo
Imports CI = System.Reflection.ConstructorInfo
Imports FI = System.Reflection.FieldInfo
Imports BF = System.Reflection.BindingFlags
Imports T = System.Type
Imports S = System

Namespace Emit

    ''' <summary>
    ''' Beinhaltet bei <see cref="Global.System.Reflection.Emit"/> häufig verwendete Typen und Member.
    ''' Gegliedert nach dem .Net-Framework.
    ''' </summary>
    Public NotInheritable Class PredefinedMembers
        Private Sub New()
        End Sub

        Public NotInheritable Class System
            Private Sub New()
            End Sub

            Public NotInheritable Class Type
                Private Sub New()
                End Sub

                Public Shared ReadOnly Type As T = GetType(S.Type)
                Public Shared ReadOnly GetTypeFromHandle_RuntimeTypeHandle As MI = Type.GetMethod("GetTypeFromHandle", BF.Public Or BF.Static, Nothing, {GetType(S.RuntimeTypeHandle)}, Nothing)
            End Class

            Public NotInheritable Class [Delegate]
                Private Sub New()
                End Sub

                Public Shared ReadOnly Type As T = GetType(S.Delegate)
                Public Shared ReadOnly Combine_Delegate_Delegate As MI = Type.GetMethod("Combine", BF.Public Or BF.Static, Nothing, {Type, Type}, Nothing)
                Public Shared ReadOnly Remove_Delegate_Delegate As MI = Type.GetMethod("Remove", BF.Public Or BF.Static, Nothing, {Type, Type}, Nothing)
            End Class

            Public NotInheritable Class EventHandler
                Private Sub New()
                End Sub

                Public Shared ReadOnly Type As T = GetType(S.EventHandler)
                Public Shared ReadOnly Constructor_Object_IntPtr As CI = Type.GetConstructor({GetType(S.Object), GetType(S.IntPtr)})
                Public Shared ReadOnly Invoke_Object_EventArgs As MI = Type.GetMethod("Invoke", BF.Public Or BF.Instance, Nothing, {GetType(S.Object), GetType(S.EventArgs)}, Nothing)
            End Class

            Public NotInheritable Class EventArgs
                Private Sub New()
                End Sub

                Public Shared ReadOnly Type As T = GetType(S.EventArgs)
                Public Shared ReadOnly Empty As FI = Type.GetField("Empty", Reflection.BindingFlags.Public Or Reflection.BindingFlags.Static)
            End Class

            Public NotInheritable Class [Object]
                Private Sub New()
                End Sub

                Public Shared ReadOnly Type As T = GetType(S.Object)
                Public Shared ReadOnly Constructor As CI = Type.GetConstructor(S.Type.EmptyTypes)
                Public Shared Shadows ReadOnly ToString As MI = Type.GetMethod("ToString", Reflection.BindingFlags.Public Or Reflection.BindingFlags.Instance, Nothing, S.Type.EmptyTypes, Nothing)
                Public Shared ReadOnly Equals_Object_Object As MI = Type.GetMethod("Equals", Reflection.BindingFlags.Public Or Reflection.BindingFlags.Static, Nothing, {GetType(S.Object), GetType(S.Object)}, Nothing)
            End Class

            Public NotInheritable Class [String]
                Private Sub New()
                End Sub

                Public Shared ReadOnly Type As T = GetType(Global.System.String)
                Public Shared ReadOnly Compare_String_String As MI = Type.GetMethod("Compare", BF.Public Or BF.Static, Nothing, {Type, Type}, Nothing)
                Public Shared ReadOnly Concat_String_String As MI = Type.GetMethod("Concat", BF.Public Or BF.Static, Nothing, {Type, Type}, Nothing)
                Public Shared ReadOnly Concat_String_String_String As MI = Type.GetMethod("Concat", BF.Public Or BF.Static, Nothing, {Type, Type, Type}, Nothing)
            End Class

            Public NotInheritable Class Console
                Private Sub New()
                End Sub

                Public Shared ReadOnly Type As T = GetType(Global.System.Console)
                Public Shared ReadOnly WriteLine_String As MI = Type.GetMethod("WriteLine", BF.Public Or BF.Static, Nothing, {GetType(Global.System.String)}, Nothing)
                'Public Shared ReadOnly WriteLine_String_ObjectArgs As MI = Type.GetMethod("WriteLine", BF.Public Or BF.Static, Nothing, {GetType(Global.System.String), GetType(S.Object)}, Nothing)
                Public Shared ReadOnly ReadLine As MI = Type.GetMethod("ReadLine", BF.Public Or BF.Static, Nothing, Type.EmptyTypes, Nothing)
                Public Shared ReadOnly Beep As MI = Type.GetMethod("Beep", BF.Public Or BF.Static, Nothing, Type.EmptyTypes, Nothing)
                Public Shared ReadOnly Beep_Int32 As MI = Type.GetMethod("Beep", BF.Public Or BF.Static, Nothing, {GetType(S.Int32)}, Nothing)
            End Class

            Public NotInheritable Class Runtime
                Private Sub New()
                End Sub

                Public NotInheritable Class CompilerServices
                    Private Sub New()
                    End Sub

                    Public NotInheritable Class RuntimeHelpers
                        Private Sub New()
                        End Sub

                        Public Shared ReadOnly Type As T = GetType(S.Runtime.CompilerServices.RuntimeHelpers)
                        Public Shared ReadOnly GetObjectValue_Object As MI = GetType(S.Runtime.CompilerServices.RuntimeHelpers).GetMethod("GetObjectValue", BF.Public Or BF.Static, Nothing, {GetType(S.Object)}, Nothing)
                    End Class

                End Class

            End Class

            Public NotInheritable Class Windows
                Private Sub New()
                End Sub

                Public NotInheritable Class Forms
                    Private Sub New()
                    End Sub

                    Public NotInheritable Class MessageBox
                        Private Sub New()
                        End Sub

                        Public Shared ReadOnly Type As T = GetType(S.Windows.Forms.MessageBox)
                        Public Shared ReadOnly Show_String As MI = Type.GetMethod("Show", Reflection.BindingFlags.Public Or Reflection.BindingFlags.Static, Nothing, {GetType(String)}, Nothing)
                    End Class

                End Class

            End Class

        End Class

    End Class

End Namespace
