﻿
Imports System.Windows.Forms

Namespace Forms

    ''' <summary>
    ''' Eine abgeänderte Form der HotKey-Klasse von ErfinderDesRades (siehe http://www.vb-paradise.de/allgemeines/sourcecode-austausch/57004-registerhotkey-wrapper/).
    ''' </summary>
    Public Class Hotkey
        Inherits System.Windows.Forms.NativeWindow
        Implements IDisposable

        ''' <summary>
        ''' Die Windows-Message für HotKeys.
        ''' </summary>
        Public Const Wm_HotKey As Integer = &H312
        'Public Const WinKey As Keys = DirectCast(Keys.Alt << 1, Keys)

        ''' <summary>
        ''' Wird ausgelöst, wenn sich der Wert der <see cref="KeyCode"/>-Eigenschaft ändert.
        ''' </summary>
        Public Event KeyCodeChanged()

        ''' <summary>
        ''' Wird ausgelöst, wenn die Tastenkombination gedrückt wurde.
        ''' </summary>
        Public Event HotKeyPressed()

        Dim _KeyCode As Keys
        ''' <summary>
        ''' Die Tastenkombination, auf die reagiert werden soll. Wird <see cref="Keys.None"/> angegeben, wird auf keine Tastenkombination reagiert.
        ''' </summary>
        Public Property KeyCode() As Keys
            <DebuggerStepThrough()> _
            Get
                Return _KeyCode
            End Get
            <DebuggerStepThrough()> _
            Set(ByVal value As Keys)
                If Not _KeyCode = value Then
                    If Registered Then
                        Unregister()
                    End If
                    If value = Keys.None OrElse TryRegister(value) Then
                        RaiseEvent KeyCodeChanged()
                    End If
                End If
            End Set
        End Property

        Public Property EnableRaisingEvent As Boolean = True

        <DebuggerStepThrough()> _
        Public Sub New()
            Me.CreateHandle(New CreateParams)
        End Sub

        <DebuggerStepThrough()> _
        Public Sub New(NewKeyCode As Keys)
            Me.New()
            KeyCode = NewKeyCode
        End Sub

        Dim Registered As Boolean = False

        <DebuggerStepThrough()> _
        Private Function TryRegister(NewKeyCode As Keys) As Boolean
            Const MOD_ALT = &H1
            Const MOD_CONTROL = &H2
            Const MOD_SHIFT = &H4
            Dim Modifier = 0
            If NewKeyCode.HasFlag(Keys.Alt) Then
                Modifier = Modifier Or MOD_ALT
            End If
            If NewKeyCode.HasFlag(Keys.Control) Then
                Modifier = Modifier Or MOD_CONTROL
            End If
            If NewKeyCode.HasFlag(Keys.Shift) Then
                Modifier = Modifier Or MOD_SHIFT
            End If
            If WinApi.HotKeys.RegisterHotKey(Me.Handle, &HAFFF, Modifier, NewKeyCode And Keys.KeyCode) Then
                _KeyCode = NewKeyCode
                Registered = True
                Return True
            End If
            Return False
        End Function

        <DebuggerStepThrough()> _
        Private Sub Unregister()
            Dim foo = WinApi.HotKeys.UnregisterHotKey(Me.Handle, _KeyCode)
            _KeyCode = Keys.None
            Registered = False
        End Sub

        <DebuggerStepThrough()> _
        Protected Overrides Sub WndProc(ByRef m As System.Windows.Forms.Message)
            If m.Msg = Wm_HotKey Then
                If _EnableRaisingEvent Then
                    RaiseEvent HotKeyPressed()
                End If
            End If
            MyBase.WndProc(m)
        End Sub

        <DebuggerStepThrough()> _
        Public Sub Dispose() Implements IDisposable.Dispose
            If Not Me.Handle = IntPtr.Zero Then
                If Registered Then
                    Unregister()
                End If
                Me.ReleaseHandle()
            End If
        End Sub

    End Class

End Namespace
