﻿
Imports System.Reflection
Imports System.Reflection.Emit
Imports OnUtils.Emit.IL
Imports OnUtils.Emit.Macros

''' <summary>
''' Beinhaltet Hilfsfunktionen, die keine Extensions sind.
''' </summary>
Public NotInheritable Class Helpers
    Private Sub New()
    End Sub

    ''' <summary>
    ''' Möglicherweise unnötig. Siehe <see cref="System.IO.Path.GetFullPath"/>.
    ''' Gibt einen absoluten Pfad zurück, der durch einen relativen Pfad beschrieben wird. ".." als Ordnername wechselt in den übergeordneten Ordner.
    ''' Beispiel: "C:\Temp\Ordner1\..\..\Users\Administrator\Desktop" wird zu "C:\Users\Administrator\Desktop" aufgelöst.
    ''' Verwenden Sie diese Funktion in Verbindung mit beispielsweise <see cref="System.IO.Path"/>.Combine(System.Windows.Forms.Application.StartupPath, "..", "..", "..")
    ''' </summary>
    ''' <param name="RelativePath">Der relative Pfad, aus dem der absolute Pfad gebildet werden soll.</param>
    ''' <exception cref="ArgumentException">Der relative Pfad versucht, über den übergeordnetsten Ordner hinaus zu wechseln. Beispiel: "C:\Users\..\..\.."
    ''' "C:\.." würde "" ergeben, "C:\..\.." kann nicht aus "" hinausgehen.</exception>
    <DebuggerStepThrough()> _
    Public Shared Function GetAbsolutePath(RelativePath As String) As String
        Dim PathList As New List(Of String)
        For Each i In RelativePath.Split(System.IO.Path.DirectorySeparatorChar)
            If i = ".." Then
                If PathList.Count = 0 Then
                    Throw New ArgumentException("Es kann nicht in das übergeordnete Verzeichnis gewechselt werden, weil keines mehr existiert. Erfolgreiche Elemente: " & PathList.Count.ToString, "RelativePath")
                End If
                PathList.RemoveAt(PathList.Count - 1)
            Else
                PathList.Add(i)
            End If
        Next
        Return String.Join(System.IO.Path.DirectorySeparatorChar, PathList)
    End Function

    Private Shared GetObjectAddressMethod As DynamicMethod
    ''' <summary>
    ''' Gibt die Adresse des angegebenen Objektes zurück. Weil man's kann.
    ''' Für Details siehe auch http://stackoverflow.com/questions/4994277
    ''' </summary>
    ''' <param name="Value">Das Objekt, dessen Adresse zurückgegeben wird.</param>
    <DebuggerStepThrough()> _
    Public Shared Function GetObjectAddress(Value As Object) As IntPtr
        If GetObjectAddressMethod Is Nothing Then
            GetObjectAddressMethod = New DynamicMethod("GetObjectAddress", GetType(IntPtr), {GetType(Object)}, Assembly.GetCallingAssembly.ManifestModule)
            With GetObjectAddressMethod.GetILGenerator
                .Ldarg_0()
                .GetObjectAddress()
                .Ret()
            End With
        End If
        Dim Uncasted = GetObjectAddressMethod.Invoke(Nothing, {Value})
        Return DirectCast(Uncasted, IntPtr)
    End Function

End Class
