﻿
Namespace Results

    ''' <summary>
    ''' Kapselt eine Angabe über den Erfolg eines Vorgangs, einen Wert, wenn der Vorgang erfolgreich abgeschlossen wurde und einen Fehler, wenn der Vorgang nicht erfolgreich abgeschlossen wurde.
    ''' </summary>
    ''' <typeparam name="TValue">Der Typ des Wertes.</typeparam>
    ''' <typeparam name="TError">Der Typ des Fehlers.</typeparam>
    Public Class ValuedErroredResult(Of TValue, TError)
        Inherits Result
        Implements IValuedResult(Of TValue)
        Implements IErroredResult(Of TError)

        Private _Value As TValue
        ''' <summary>
        ''' Der vom Vorgang zurückgegebene Wert.
        ''' </summary>
        Public ReadOnly Property Value As TValue Implements IValuedResult(Of TValue).Value
            <DebuggerStepThrough()> _
            Get
                Return _Value
            End Get
        End Property

        Private _Error As TError
        ''' <summary>
        ''' Der Fehler im Vorgang.
        ''' </summary>
        Public ReadOnly Property [Error] As TError Implements IErroredResult(Of TError).Error
            <DebuggerStepThrough()> _
            Get
                Return _Error
            End Get
        End Property

        ''' <summary>
        ''' Der Vorgang wurde erfolgreich abgeschlossen; <paramref name="NewValue"/> wurde zurückgegeben.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub New(NewValue As TValue)
            MyBase.New(True)
            _Value = NewValue
        End Sub

        ''' <summary>
        ''' Der Vorgang wurde nicht erfolgreich abgeschlossen; <paramref name="NewError"/> ist der Fehler.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub New(NewError As TError)
            MyBase.New(False)
            _Error = NewError
        End Sub

        ''' <summary>
        ''' Wird verwendet wenn die Konstruktoren, die <see cref="TValue"/> und <see cref="TError"/> verwenden nicht eindeutig genug sind.
        ''' </summary>
        ''' <param name="Undefined">Gibt den Wert oder Fehler an.</param>
        ''' <param name="Success">True, wenn der Vorgang erfolgreich abgeschlossen wurde, andernfalls False.</param>
        <DebuggerStepThrough()> _
        Public Sub New(Undefined As Object, Success As Boolean)
            MyBase.New(Success)
            If Success Then
                _Value = DirectCast(Undefined, TValue)
            Else
                _Error = DirectCast(Undefined, TError)
            End If
        End Sub

    End Class

End Namespace
