﻿
Namespace Streams

    ''' <summary>
    ''' Puffert Daten und löst ein Event aus, wenn ein vollständiger Block vorhanden ist.
    ''' </summary>
    ''' <typeparam name="T">Der Typ der Daten.</typeparam>
    Public Class DataBuffer(Of T)

        ''' <summary>
        ''' Wird ausgelöst, wenn ein vollständiger Block an Daten vorhanden ist bzw. wenn ein Delimiter empfangen wurde.
        ''' </summary>
        Public Event BlockReceived As EventHandler(Of BufferDataReceivedEventArgs(Of T))
        <DebuggerStepThrough()> _
        Private Sub OnBlockReceived(Block As DataBlock(Of T))
            RaiseEvent BlockReceived(Me, New BufferDataReceivedEventArgs(Of T)(Block.ToArray()))
        End Sub

        Dim _Source As IDataBufferSource(Of T)
        ''' <summary>
        ''' Die Datenquelle des Puffers.
        ''' </summary>
        Public ReadOnly Property Source As IDataBufferSource(Of T)
            <DebuggerStepThrough()> _
            Get
                Return _Source
            End Get
        End Property

        Dim _DelimiterData As IEnumerable(Of T)
        ''' <summary>
        ''' Die Abfolge an Daten, die zwischen auszuwertenden Daten empfangen wird.
        ''' </summary>
        Public ReadOnly Property DelimiterData As IEnumerable(Of T)
            <DebuggerStepThrough()> _
            Get
                Return _DelimiterData
            End Get
        End Property

        Dim Delimiter As DataDelimiter(Of T)
        Dim CurrentBlock As DataBlock(Of T)

        ''' <summary>
        ''' Konstruktor.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub New(NewSource As IDataBufferSource(Of T), NewDelimiterData As IEnumerable(Of T))
            _Source = NewSource
            _DelimiterData = NewDelimiterData
            Delimiter = New DataDelimiter(Of T)(_Source, _DelimiterData)
            CurrentBlock = New DataBlock(Of T)
            AddHandler _Source.DataReceived, AddressOf SourceDataReceived
        End Sub

        <DebuggerStepThrough()> _
        Private Sub SourceDataReceived(sender As Object, e As BufferDataReceivedEventArgs(Of T))
            For Each i In e.Data
                CurrentBlock.Append(i)
                If Delimiter.TryMatchNext(i) Then
                    Delimiter.Reset()
                    CurrentBlock.StripEnd(Delimiter.ItemCount)
                    OnBlockReceived(CurrentBlock)
                    CurrentBlock = New DataBlock(Of T)
                End If
            Next
        End Sub

        ''' <summary>
        ''' Löst das <see cref="BlockReceived"/>-Event für die aktuell gepufferten Daten aus, auch wenn noch kein Delimiter empfangen wurde.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub Flush()
            _Source.Flush()
            If Not CurrentBlock.Count = 0 Then
                OnBlockReceived(CurrentBlock)
                CurrentBlock = New DataBlock(Of T)
            End If
        End Sub

    End Class

End Namespace
