﻿
Namespace Streams

    Friend Class DataDelimiter(Of T)

        Public ReadOnly Property MatchedElements As Integer
            <DebuggerStepThrough()> _
            Get
                Return CurrentIndex
            End Get
        End Property

        Dim _ItemCount As Integer
        Public ReadOnly Property ItemCount As Integer
            <DebuggerStepThrough()> _
            Get
                Return _ItemCount
            End Get
        End Property

        Dim Source As IDataBufferSource(Of T)
        Dim CurrentIndex As Integer = 0
        Dim Data As IEnumerable(Of T)
        Dim DataEnumerator As IEnumerator(Of T)

        <DebuggerStepThrough()> _
        Public Sub New(NewSource As IDataBufferSource(Of T), NewData As IEnumerable(Of T))
            Source = NewSource
            Data = NewData
            _ItemCount = Data.Count
            DataEnumerator = Data.GetEnumerator
            If Not DataEnumerator.MoveNext Then
                Throw New ArgumentException("Der Delimiter muss mindestens ein Element beinhalten.")
            End If
        End Sub

        <DebuggerStepThrough()> _
        Public Function TryMatchNext(Value As T) As Boolean
            If Source.CompareData(DataEnumerator.Current, Value) Then
                CurrentIndex += 1
                Return Not DataEnumerator.MoveNext
            Else
                Reset()
                Return False
            End If
        End Function

        <DebuggerStepThrough()> _
        Public Sub Reset()
            CurrentIndex = 0
            DataEnumerator.Reset()
            DataEnumerator.MoveNext()
        End Sub

    End Class

End Namespace
