﻿
Imports P = System.IO.Ports

Namespace Streams

    ''' <summary>
    ''' Eine Datenquelle zum Puffern von <see cref="P.SerialPort"/>.
    ''' </summary>
    Public Class SerialPortDataSource
        Implements IDataBufferSource(Of Byte)

        Public Event DataReceived As EventHandler(Of BufferDataReceivedEventArgs(Of Byte)) Implements IDataBufferSource(Of Byte).DataReceived

        Dim _Port As P.SerialPort
        ''' <summary>
        ''' Das Port, dessen Daten gepuffert werden.
        ''' </summary>
        Public ReadOnly Property Port As P.SerialPort
            <DebuggerStepThrough()> _
            Get
                Return _Port
            End Get
        End Property

        ''' <summary>
        ''' Konstruktor.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub New(NewPort As P.SerialPort)
            If NewPort Is Nothing Then
                Throw New ArgumentNullException("NewPort")
            End If
            _Port = NewPort
            AddHandler _Port.DataReceived, AddressOf PortDataReceived
        End Sub

        <DebuggerStepThrough()> _
        Private Sub PortDataReceived(sender As Object, e As P.SerialDataReceivedEventArgs)
            ReadPortBytes()
        End Sub

        <DebuggerStepThrough()> _
        Private Sub ReadPortBytes()
            If Not _Port.BytesToRead = 0 Then
                Dim Buffer = New Byte(_Port.BytesToRead - 1) {}
                _Port.Read(Buffer, 0, _Port.BytesToRead)
                RaiseEvent DataReceived(Me, New BufferDataReceivedEventArgs(Of Byte)(Buffer))
            End If
        End Sub

        <DebuggerStepThrough()> _
        Public Sub Flush() Implements IDataBufferSource(Of Byte).Flush
            ReadPortBytes()
        End Sub

        <DebuggerStepThrough()> _
        Public Function CompareData(Item1 As Byte, Item2 As Byte) As Boolean Implements IDataBufferSource(Of Byte).CompareData
            Return Item1 = Item2
        End Function

    End Class

End Namespace
