﻿
Imports System.Text

Namespace Streams

    ''' <summary>
    ''' Kapselt einen <see cref="StringBuilder"/> und bietet Methoden zum Einrücken von Zeilen.
    ''' </summary>
    Public Class IndentStringBuilder

        Dim _IndentLevel As Integer = 0
        ''' <summary>
        ''' Die aktuelle Einrückungstiefe.
        ''' <see cref="IndentLevel"/> * <see cref="IndentationsPerLevel"/> ergibt die Anzahl an Zeichen, mit denen eingerückt wird.
        ''' </summary>
        Public Property IndentLevel As Integer
            <DebuggerStepThrough()> _
            Get
                Return _IndentLevel
            End Get
            <DebuggerStepThrough()> _
            Set(value As Integer)
                If value < 0 Then
                    Throw New ArgumentOutOfRangeException("value", value, "Die Einrückung darf nicht negativ sein.")
                End If
            End Set
        End Property

        Dim _IndentationsPerLevel As Integer = 4
        ''' <summary>
        ''' Gibt an, wie viele Zeichen pro Einrückung verwendet werden.
        ''' Standardwert: 4
        ''' </summary>
        Public Property IndentationsPerLevel As Integer
            <DebuggerStepThrough()> _
            Get
                Return _IndentationsPerLevel
            End Get
            <DebuggerStepThrough()> _
            Set(value As Integer)
                If value < 0 Then
                    Throw New ArgumentOutOfRangeException("value", value, "Die Einrückung darf nicht negativ sein.")
                End If
                _IndentationsPerLevel = value
            End Set
        End Property

        ''' <summary>
        ''' Das Zeichen, das zum Einrücken verwendet wird.
        ''' Standardwert: Leerzeichen
        ''' </summary>
        Public Property IndentChar As Char = " "c

        Dim SB As New StringBuilder

        ''' <summary>
        ''' Erhöht die Einrückung (<see cref="IndentLevel"/>) um 1.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub Indent()
            _IndentLevel += 1
        End Sub

        ''' <summary>
        ''' Verringert die Einrückung (<see cref="IndentLevel"/>) um 1.
        ''' </summary>
        ''' <exception cref="ArgumentOutOfRangeException"><see cref="IndentLevel"/> ist bereits 0.</exception>
        <DebuggerStepThrough()> _
        Public Sub UnIndent()
            If _IndentLevel = 0 Then
                Throw New InvalidOperationException("Es kann nicht weiter ausgerückt werden.")
            End If
            _IndentLevel -= 1
        End Sub

        ''' <summary>
        ''' Hängt nur den Einrückungs-String an.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub AppendIndentation()
            SB.Append(New String(_IndentChar, _IndentLevel * _IndentationsPerLevel))
        End Sub

        ''' <summary>
        ''' Hängt <paramref name="Value"/> ohne Einrückung und ohne Zeilenumbruch an.
        ''' </summary>
        ''' <param name="Value">Der anzuhängende String.</param>
        <DebuggerStepThrough()> _
        Public Sub Append(Value As String)
            SB.Append(Value)
        End Sub

        ''' <summary>
        ''' Hängt einen Zeilenumbruch an.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Sub AppendLine()
            SB.AppendLine()
        End Sub

        ''' <summary>
        ''' Hängt den Einrückungs-String, <paramref name="Value"/> und einen Zeilenumbruch an.
        ''' </summary>
        ''' <param name="Value"></param>
        <DebuggerStepThrough()> _
        Public Sub AppendLine(Value As String)
            AppendIndentation()
            SB.AppendLine(Value)
        End Sub

        ''' <summary>
        ''' Gibt den Inhalt des StringBuilders zurück.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Overrides Function ToString() As String
            Return SB.ToString
        End Function

    End Class

End Namespace
