﻿
Namespace WinApi

    ''' <summary>
    ''' Eine Kombination von 3 Farbwerten (Rot, Grün und Blau) und einem Alphakanal, der die Transparenz angibt.
    ''' Bei Bedarf kann mit <see cref="DwmColor.ToFormsColor"/> und <see cref="DwmColor.ToWpfColor"/> konvertiert werden.
    ''' </summary>
    Public Structure DwmColor

        Private _A As Byte
        ''' <summary>
        ''' Alpha
        ''' </summary>
        Public ReadOnly Property A As Byte
            <DebuggerStepThrough()> _
            Get
                Return _A
            End Get
        End Property

        Private _R As Byte
        ''' <summary>
        ''' Alpha
        ''' </summary>
        Public ReadOnly Property R As Byte
            <DebuggerStepThrough()> _
            Get
                Return _R
            End Get
        End Property

        Private _G As Byte
        ''' <summary>
        ''' Alpha
        ''' </summary>
        Public ReadOnly Property G As Byte
            <DebuggerStepThrough()> _
            Get
                Return _G
            End Get
        End Property

        Private _B As Byte
        ''' <summary>
        ''' Alpha
        ''' </summary>
        Public ReadOnly Property B As Byte
            <DebuggerStepThrough()> _
            Get
                Return _B
            End Get
        End Property

        ''' <summary>
        ''' Konstruktor.
        ''' </summary>
        ''' <param name="NewA">Der Alpha-Kanal.</param>
        ''' <param name="NewR">Der Rot-Anteil.</param>
        ''' <param name="NewG">Der Grün-Anteil.</param>
        ''' <param name="NewB">Der Blau-Anteil.</param>
        <DebuggerStepThrough()> _
        Public Sub New(NewA As Byte, NewR As Byte, NewG As Byte, NewB As Byte)
            _A = NewA
            _R = NewR
            _G = NewG
            _B = NewB
        End Sub

        ''' <summary>
        ''' Konstruktor.
        ''' </summary>
        ''' <param name="Argb">Die Farbwerte als Vorzeichenlose 32-Bit Ganzzahl. Die 8 höchstwertigen Bits beinhalten den Alphakanal, die 8 niederwertigsten den Blauanteil.</param>
        <DebuggerStepThrough()> _
        Public Sub New(Argb As UInt32)
            _A = CByte(Argb >> 24 And 255)
            _R = CByte(Argb >> 16 And 255)
            _G = CByte(Argb >> 8 And 255)
            _B = CByte(Argb And 255)
        End Sub

        ''' <summary>
        ''' Gibt eine <see cref="System.Drawing.Color"/> mit den gleichen Farbwerten zurück.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Function ToFormsColor() As System.Drawing.Color
            Return System.Drawing.Color.FromArgb(_A, _R, _G, _B)
        End Function

        ''' <summary>
        ''' Gibt eine <see cref="System.Windows.Media.Color"/> mit den gleichen Farbwerten zurück.
        ''' </summary>
        <DebuggerStepThrough()> _
        Public Function ToWpfColor() As System.Windows.Media.Color
            Return System.Windows.Media.Color.FromArgb(_A, _R, _G, _B)
        End Function

    End Structure

End Namespace
