﻿
Namespace WinApi

    ''' <summary>
    ''' Beinhaltet WinApi-Aufrufe, die mit der Funktion SetWindowPos() zu tun haben.
    ''' </summary>
    Public NotInheritable Class SetWindowPos
        Private Sub New()
        End Sub

        ''' <summary>
        ''' Setzt ein Fenster, dem das in <paramref name="hWnd">hWnd</paramref> angegebene Handle gehört, auf eine in <paramref name="X">X</paramref> und <paramref name="Y">Y</paramref> angegebene Position und die in <paramref name="hWndInsertAfter">hWndInsertAfter</paramref> angegebene Ebene, unter Berücksichtigung der in <paramref name="uFlags">uFlags</paramref> angegebenen Bedingungen.
        ''' </summary>
        ''' <param name="hWnd">Das Handle des Fensters.</param>
        ''' <param name="hWndInsertAfter">Die Angabe darüber, in welche Z-Ebene das Fenster verschoben werden soll.</param>
        ''' <param name="X">Die X-Position des Fensters.</param>
        ''' <param name="Y">Die Y-Position des Fensters.</param>
        ''' <param name="cx">Die Breite des Fensters.</param>
        ''' <param name="cy">Die Höhe des Fensters.</param>
        ''' <param name="uFlags">Zusätzliche Parameter. Werden mit Or kombiniert.</param>
        <System.Runtime.InteropServices.DllImport("user32.dll", SetLastError:=True)> _
        Private Shared Function SetWindowPos(hWnd As IntPtr, hWndInsertAfter As IntPtr, X As Integer, Y As Integer, cx As Integer, cy As Integer, uFlags As UInteger) As Boolean
        End Function

        ''' <summary>
        ''' Gibt Bedingungen für die SetWindowPos-Funktion an.
        ''' </summary>
        Public Enum Flags As UInteger
            ''' <summary>
            ''' If the calling thread and the thread that owns the window are attached to different input queues, the system posts the request to the thread that owns the window. This prevents the calling thread from blocking its execution while other threads process the request. 
            ''' </summary>
            Swp_AsyncWindowPos = &H4000
            ''' <summary>
            ''' Prevents generation of the Wm_SyncPaint message. 
            ''' </summary>
            Swp_DeferErase = &H2000
            ''' <summary>
            ''' Draws a frame (defined in the window's class description) around the window.
            ''' </summary>
            Swp_DrawFrame = &H20
            ''' <summary>
            ''' Applies new frame styles set using the SetWindowLong function. Sends a Wm_NCCalcSize message to the window, even if the window's size is not being changed. If this flag is not specified, Wm_NCCalcSize is sent only when the window's size is being changed.
            ''' </summary>
            Swp_FrameChanged = &H20
            ''' <summary>
            ''' Hides the window.
            ''' </summary>
            Swp_HideWindow = &H80
            ''' <summary>
            ''' Does not activate the window. If this flag is not set, the window is activated and moved to the top of either the topmost or non-topmost group (depending on the setting of the hWndInsertAfter parameter).
            ''' </summary>
            Swp_NoActivate = &H10
            ''' <summary>
            ''' Discards the entire contents of the client area. If this flag is not specified, the valid contents of the client area are saved and copied back into the client area after the window is sized or repositioned.
            ''' </summary>
            Swp_NoCopyBits = &H100
            ''' <summary>
            ''' Retains the current position (ignores X and Y parameters).
            ''' </summary>
            Swp_NoMove = &H2
            ''' <summary>
            ''' Does not change the owner window's position in the Z order.
            ''' </summary>
            Swp_NoOwnerZOrder = &H2
            ''' <summary>
            ''' Does not redraw changes. If this flag is set, no repainting of any kind occurs. This applies to the client area, the nonclient area (including the title bar and scroll bars), and any part of the parent window uncovered as a result of the window being moved. When this flag is set, the application must explicitly invalidate or redraw any parts of the window and parent window that need redrawing.
            ''' </summary>
            Swp_NoRedraw = &H8
            ''' <summary>
            ''' Same as the Swp_NoOwnerZOrder flag.
            ''' </summary>
            Swp_NoReposition = &H200
            ''' <summary>
            ''' Prevents the window from receiving the Wm_WindowPosChanging message.
            ''' </summary>
            Swp_NoSendChanging = &H400
            ''' <summary>
            ''' Retains the current size (ignores the width and height parameters).
            ''' </summary>
            Swp_NoSize = &H1
            ''' <summary>
            ''' Retains the current Z order (ignores the hWndInsertAfter parameter).
            ''' </summary>
            Swp_NoZOrder = &H40
            ''' <summary>
            ''' Displays the window.
            ''' </summary>
            Swp_ShowWindow = &H40
        End Enum

        ''' <summary>
        ''' Gibt an, in welche Z-Ebene das Fenster verschoben wird.
        ''' </summary>
        Public Enum ZPosition As Integer
            ''' <summary>
            ''' Verschiebt das Fenster ganz nach hinten. Handelt es sich bei dem Fenster um ein TopMost-Fenster, wird TopMost auf False gesetzt.
            ''' </summary>
            TotalBottom = 1
            ''' <summary>
            ''' Verschiebt das Fenster vor alle Nicht-TopMost-Fenster und hinter alle TopMost-Fenster.
            ''' </summary>
            BehindTopMost = -2
            ''' <summary>
            ''' Verschiebt das Fenster vor alle anderen Fenster. Wird ein anderes Fenster aktiviert, geht dieser Effekt verloren.
            ''' </summary>
            TotalFront = 0
            ''' <summary>
            ''' Verschiebt das Fenster vor alle anderen Fenster und fixiert es in dieser Ebene. Wird ein anderes Fenster aktiviert, bleibt es im Fordergrund.
            ''' </summary>
            TotalFrontFixed = -1
        End Enum

        ''' <summary>
        ''' Setzt ein Fenster, dem das in <paramref name="hWnd">hWnd</paramref> angegebene Handle gehört, auf eine in <paramref name="X">X</paramref> und <paramref name="Y">Y</paramref> angegebene Position und die in <paramref name="hWndInsertAfter">hWndInsertAfter</paramref> angegebene Ebene, unter Berücksichtigung der in <paramref name="uFlags">uFlags</paramref> angegebenen Bedingungen.
        ''' </summary>
        ''' <param name="WindowHandle">Das Handle des Fensters.</param>
        ''' <param name="ZPosition">Die Angabe darüber, in welche Z-Ebene das Fenster verschoben werden soll.</param>
        ''' <param name="X">Die X-Position des Fensters.</param>
        ''' <param name="Y">Die Y-Position des Fensters.</param>
        ''' <param name="Width">Die Breite des Fensters.</param>
        ''' <param name="Height">Die Höhe des Fensters.</param>
        ''' <param name="Flags">Zusätzliche Parameter. Werden mit Or kombiniert.</param>
        <DebuggerStepThrough()> _
        Public Shared Sub SetWindowPos(WindowHandle As IntPtr, ZPosition As ZPosition, X As Integer, Y As Integer, Width As Integer, Height As Integer, Flags As Flags)
            SetWindowPos(WindowHandle, New IntPtr(ZPosition), X, Y, Width, Height, Flags)
        End Sub

    End Class

End Namespace
