﻿
Imports System.Windows.Input

Namespace Wpf

    ''' <summary>
    ''' Stellt ein ICommand dar, das einen Delegaten ausführt.
    ''' Der Delegat kann entweder keine oder ein Argument vom Typ Object entgegennehmen.
    ''' Die <see cref="DelegateCommand.CanExecute"/>-Funktion gibt immer True zurück, kann jedoch überschrieben werden.
    ''' </summary>
    Public Class DelegateCommand
        Implements ICommand

        ''' <summary>
        ''' Siehe <see cref="ICommand.CanExecuteChanged"/>.
        ''' </summary>
        Public Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
        ''' <summary>
        ''' Löst das <see cref="CanExecuteChanged"/>-Event aus
        ''' </summary>
        <DebuggerStepThrough()> _
        Protected Sub OnCanExecuteChanged()
            RaiseEvent CanExecuteChanged(Me, EventArgs.Empty)
        End Sub

        Dim HasParameter As Boolean
        Dim TargetWithParameter As Action(Of Object)
        Dim TargetWithoutParameter As Action

        <DebuggerStepThrough()> _
        Public Sub New(NewTarget As Action(Of Object))
            HasParameter = True
            TargetWithParameter = NewTarget
        End Sub

        <DebuggerStepThrough()> _
        Public Sub New(NewTarget As Action)
            HasParameter = False
            TargetWithoutParameter = NewTarget
        End Sub

        <DebuggerStepThrough()> _
        Public Overridable Function CanExecute(Parameter As Object) As Boolean Implements ICommand.CanExecute
            Return True
        End Function

        <DebuggerStepThrough()> _
        Public Overridable Sub Execute(Parameter As Object) Implements ICommand.Execute
            If HasParameter Then
                TargetWithParameter(Parameter)
            Else
                TargetWithoutParameter()
            End If
        End Sub

    End Class

End Namespace
