﻿
Imports System.Windows.Documents

Namespace Wpf

    ''' <summary>
    ''' Enthält Hilfsfunktionen zum Erstellen von Inlines für <see cref="FormatTextBlock"/>.
    ''' </summary>
    Public NotInheritable Class TextInlines
        Private Sub New()
        End Sub

        Private Shared ReadOnly _Empty As Inline() = New Inline() {}
        ''' <summary>
        ''' Ein leeres Array von <see cref="Inline"/>.
        ''' </summary>
        Public Shared ReadOnly Property Empty As Inline()
            <DebuggerStepThrough()> _
            Get
                Return _Empty
            End Get
        End Property

        ''' <summary>
        ''' Eine Funktion, die anhand eines übergebenen Objektes eine Auflistung von <see cref="Inline"/>-Objektnen zurückgibt, die das Objekt repräsentieren.
        ''' Die <see cref="Create"/>-Funktionen werden dadurch erweitert.
        ''' </summary>
        Public Shared Property InlineSelector As Func(Of Object, IEnumerable(Of Inline))

        ''' <summary>
        ''' Gibt eine List von <see cref="Inline"/>-Objekten zurück, die die angegebenen Objekte repräsentieren.
        ''' </summary>
        ''' <param name="Items">Das ParamArray von Objekten.</param>
        <DebuggerStepThrough()> _
        Public Shared Function Create(ParamArray Items As Object()) As IEnumerable(Of Inline)
            Return Create(DirectCast(Items, IEnumerable(Of Object)))
        End Function

        ''' <summary>
        ''' Gibt eine List von <see cref="Inline"/>-Objekten zurück, die die angegebenen Objekte repräsentieren.
        ''' </summary>
        ''' <param name="Items">Die Auflistung von Objekten.</param>
        <DebuggerStepThrough()> _
        Public Shared Function Create(Items As IEnumerable(Of Object)) As IEnumerable(Of Inline)
            Dim Temp As New List(Of Inline)
            For Each i In Items
                If TypeOf i Is String Then
                    Temp.Add(New Run(DirectCast(i, String)))
                ElseIf TypeOf i Is Inline Then
                    Temp.Add(DirectCast(i, Inline))
                ElseIf TypeOf i Is IEnumerable(Of Inline) Then
                    Temp.AddRange(DirectCast(i, IEnumerable(Of Inline)))
                Else
                    If _InlineSelector Is Nothing Then
                        Throw New ArgumentException("Unbehandelter Typ '" & i.GetType.Name & "' für TextInlines.Create.", "Items")
                    Else
                        Temp.AddRange(_InlineSelector(i))
                    End If
                End If
            Next
            Return Temp
        End Function

    End Class

End Namespace
