﻿
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input

Namespace Wpf

    ''' <summary>
    ''' Ein Button, der beim Klick, falls vorhanden, den zugeordneten ToolTip öffnet.
    ''' ToolTip muss auf ein Objekt des Typs <see cref="ToolTip"/> festgelegt werden, andernfalls wird eine <see cref="InvalidToolTipException"/> ausgelöst.
    ''' </summary>
    Public Class ToolTipButton
        Inherits Button

        <DebuggerStepThrough()> _
        Public Sub New()
            AddHandler Me.Click, AddressOf OpenToolTip
            AddHandler Me.MouseLeave, AddressOf CloseToolTip
        End Sub

        <DebuggerStepThrough()> _
        Private Function CheckToolTipValid() As Boolean
            If Me.ToolTip Is Nothing Then
                Return False
            End If
            If TypeOf Me.ToolTip Is ToolTip Then
                Return True
            End If
            Throw New InvalidToolTipException(Me.ToolTip)
        End Function

        <DebuggerStepThrough()> _
        Private Sub OpenToolTip(sender As Object, e As RoutedEventArgs)
            If CheckToolTipValid() Then
                With DirectCast(Me.ToolTip, ToolTip)
                    If Not .IsOpen Then
                        .IsOpen = True
                    End If
                End With
            End If
        End Sub

        <DebuggerStepThrough()> _
        Private Sub CloseToolTip(sender As Object, e As MouseEventArgs)
            If CheckToolTipValid() Then
                With DirectCast(Me.ToolTip, ToolTip)
                    If .IsOpen Then
                        .IsOpen = False
                    End If
                End With
            End If
        End Sub

    End Class

End Namespace
